#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000UL
#include <util/delay.h>

uint8_t waitAndRead(uint8_t withTimeout);
void init_UART();
void init_Timer();
void sendByte(uint8_t c);
volatile uint8_t timeout = 0;

void init_UART(void)
{
	UBRRL = 103; //9600 bps @ 16 MHz crystal
	UCSRB = _BV(TXEN); //EX enable
	UCSRC = _BV(URSEL)|_BV(UCSZ1)|_BV(UCSZ0); //n,8,1
}

void init_Timer() {
	TIMSK = _BV(TOIE0);               // Int T0 Overflow enabled
	TCCR0 = _BV(CS00)|_BV(CS01);      // CLK/64
}

ISR(TIMER0_OVF_vect)
{ 
	timeout = 1;
}

int main() {
	init_UART();
	init_Timer();
	sei();
	uint8_t oldControl = 0;
	while(1) {
		uint8_t ret = 0;
		uint16_t val = 0;
		for (uint8_t i=0; i<14; i++) {
			val <<= 1;
			ret = waitAndRead(i != 0);
			if (ret == 2) break;
			val |= ret;
		}
		if (ret == 2) continue;
		uint8_t command = val & 0b00111111;
		uint8_t address = (val >> 6) & 0b00011111;
		uint8_t control = (val >> 11) & 1;
		sendByte(command);
		sendByte(address);
		sendByte(control != oldControl);
		oldControl = control;
	}
	return 0;
}

uint8_t waitAndRead(uint8_t withTimeout) {
	_delay_ms(1);
	uint8_t state = PINA & _BV(PINA0);
	uint8_t oldState = state;
	if (withTimeout) {
		TCNT0 = 0;
		timeout = 0;
		while ((state == oldState) && (timeout == 0)) state = PINA & _BV(PINA0);		
		if (timeout == 1) return 2;
	} else {
		while (state == oldState) state = PINA & _BV(PINA0);
	}
	if (state) return 0; else return 1;
}

void sendByte(uint8_t c) {
	while(!(UCSRA & _BV(UDRE)));
	UDR = c;
}
