#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000U
#include <util/delay.h>

volatile uint8_t state;
volatile uint8_t connected = 0;
volatile uint8_t firstLf = 0;
volatile uint8_t firstChar = 0;

void init_UART (void)
{
	UBRRL = 51; //19200 bps @ 16 MHz crystal
	UCSRB = _BV(RXEN)|_BV(TXEN)|_BV(RXCIE); //EX enable, RX int enable
	UCSRC = _BV(URSEL)|_BV(UCSZ1)|_BV(UCSZ0); //n,8,1
}

ISR(USART_RXC_vect) {
	uint8_t data = UDR;
	if(connected) {
		if (data=='a') {
			PORTA |= _BV(PINA0);
		}
		if (data=='b') {
			PORTA &= ~_BV(PINA0);
		}
		if (data=='r') {
			while(!(UCSRA & _BV(UDRE)));
			if(state==0) UDR = 'P'; else UDR = 'N';
		}		
	}
	if(firstLf) {
		if(!firstChar) firstChar = data;
	}
	if (data=='\n'){
		if (firstLf) {
			firstLf = 0;
			if (firstChar == 'C') {
				connected = 1;
			}
			if (firstChar == 'D') {
				connected = 0;
			}
			firstChar = 0;
		} else firstLf = 1;
	}
}

int main() {
	uint8_t stateOld = 3;
	init_UART();
	sei();
	DDRA |= _BV(DDA0)|_BV(DDA2);
	PORTA |= _BV(PINA1);
	while(1){
		while(!connected);
		PORTA |= _BV(PINA2);
		while(connected) {
			state = PINA & _BV(PINA1);
			if (state!=stateOld) {
				stateOld = state;
				while(!(UCSRA & _BV(UDRE)));
				if(state==0) UDR = 'P'; else UDR = 'N';
				_delay_ms(50);
			}
		}
		PORTA &= ~_BV(PINA2);
	}	
}
