﻿#include <avr/interrupt.h>
#include <avr/io.h>

#define F_CPU 16000000UL
#define BAUDRATE 9600UL

enum SoftwareUartStateRx {IDLE_RX, RECEIVE, RECEIVE_STOP};
enum SoftwareUartStateTx {IDLE_TX, TRANSMIT, TRANSMIT_STOP};
volatile enum SoftwareUartStateRx softwareUartStateRx = IDLE_RX;
volatile enum SoftwareUartStateTx softwareUartStateTx = IDLE_TX;
volatile uint8_t cyclesCounterRx, cyclesCounterTx;
volatile uint8_t softwareUartBitsToSend;
volatile uint16_t softwareUartOutputData;
volatile uint8_t softwareUartInputData;
volatile uint8_t softwareUartNewData = 0;

#define PORT_RX PIND //RX port
#define PIN_RX PIND2 //pin of RX port
#define PORT_TX PORTD //TX port
#define PIN_TX PORTD3 //pin of TX port
#define DDR_TX DDRD //data direction register for TX port

void init_Timer() {
	TIMSK = _BV(OCIE1A); //interrupt on compare
	TCCR1B = _BV(WGM12)|_BV(CS10); //no clock division
	OCR1A = F_CPU / BAUDRATE / 3 - 1; //maximum value for timer's counter
}

ISR(TIMER1_COMPA_vect)
{ 
	static uint8_t bitsToReceive;
	static uint8_t inData;
	
	if (softwareUartStateTx == TRANSMIT) {
		if (--cyclesCounterTx == 0) {
			if(softwareUartOutputData & 1) PORT_TX |= _BV(PIN_TX); else PORT_TX &= ~_BV(PIN_TX); //data bit
			softwareUartOutputData >>= 1;
			cyclesCounterTx = 3;
			if (--softwareUartBitsToSend == 0) {
				softwareUartStateTx = TRANSMIT_STOP;
			}
		}
	}
	
	if (softwareUartStateTx == TRANSMIT_STOP) {
		if (--cyclesCounterTx == 0) {
			softwareUartStateTx = IDLE_TX;
		}
	}				
	
	if ((softwareUartStateRx == IDLE_RX) && !(PORT_RX & _BV(PIN_RX))) {
		cyclesCounterRx = 4;
		softwareUartStateRx = RECEIVE;
		bitsToReceive = 8;
		inData = 0;
	}
	
	if (softwareUartStateRx == RECEIVE) {
		if (--cyclesCounterRx == 0) {
			inData >>= 1;
			if (PORT_RX & _BV(PIN_RX)) inData |= 0x80;
			cyclesCounterRx = 3;
			if (--bitsToReceive == 0) {
				softwareUartStateRx = RECEIVE_STOP;
				softwareUartInputData = inData;
				softwareUartNewData = 1;
				//processIncomingData();
			}
		}			
	}
	
	if (softwareUartStateRx == RECEIVE_STOP) {
		if (--cyclesCounterRx == 0) {
			softwareUartStateRx = IDLE_RX;
		}
	}
}

void softwareUartSend(uint16_t data) {
	while(softwareUartStateTx != IDLE_TX);
	cyclesCounterTx = 1;
	softwareUartBitsToSend = 10;
	softwareUartOutputData = (data << 1) | 0b1000000000;
	softwareUartStateTx = TRANSMIT;
}

int main(void)
{
	init_Timer();
	sei();
	DDR_TX = _BV(PIN_TX);
	PORT_TX |= _BV(PIN_TX);
    while(1)
    {
		softwareUartSend(0x55);
    }
}
