#include <avr/interrupt.h>
#include <avr/io.h>

void sendByte(char c);
void init_UART(void);
void init_ADC(void);
void getADC();

void init_ADC(void) { 
	// Internal 2.56V voltage source as reference voltage for ADC
	ADMUX |= _BV(REFS0)|_BV(REFS1); 
	// Select ADC channel (ADC1) 
	ADMUX |= _BV(MUX0);
	// Enable conversion
	ADCSRA |= _BV(ADEN); 
	// Set ADC prescaler
	ADCSRA |= _BV(ADPS0)|_BV(ADPS1)|_BV(ADPS2); 
}

void init_UART (void) {
	UBRRL = 103;
	UCSRB = _BV(TXEN)|_BV(RXEN)|_BV(RXCIE);
	UCSRC = _BV(URSEL)|_BV(UCSZ1)|_BV(UCSZ0);
	sei();
}

ISR(USART_RXC_vect) {
	uint8_t data = UDR;
	if (data=='x') {
		getADC();
		sendByte(ADCL);
		sendByte(ADCH);
	}
}

void getADC() {
	ADCSRA |= _BV(ADSC); 
	while(ADCSRA & _BV(ADSC)) {}; 
}

int main(void) 
{ 
	init_UART();
	init_ADC();
	while(1);
}

void sendByte(char c) {
	while(!(UCSRA & _BV(UDRE)));
	UDR = c;
}
