#include <avr/io.h>
#include <stdio.h>

void sendByte(char c);
void init_UART(void);
void init_ADC(void);
uint16_t getADC();
void hexStr(char * buf, uint16_t val);

void init_ADC(void) { 
	// Internal 2.56V voltage source as reference voltage for ADC
	ADMUX |= _BV(REFS0)|_BV(REFS1); 
	// Select ADC channel (ADC1) 
	ADMUX |= _BV(MUX0);
	// Enable conversion
	ADCSRA |= _BV(ADEN); 
	// Set ADC prescaler
	ADCSRA |= _BV(ADPS0)|_BV(ADPS1)|_BV(ADPS2); 
}

void init_UART (void) {
	UBRRL = 103;
	UCSRB = _BV(TXEN);
	UCSRC = _BV(URSEL)|_BV(UCSZ1)|_BV(UCSZ0);
}

uint16_t getADC() {
	ADCSRA |= _BV(ADSC); 
	while(ADCSRA & _BV(ADSC)) {};
	return ADCW;
}

int main(void) 
{ 
	init_UART();
	init_ADC();
	char buf[5];
	while(1) {
		hexStr(buf, getADC());
		sendByte(0xff);
		for (int i=0; i<4; i++) sendByte(buf[i]);
	}
}

void sendByte(char c) {
	while(!(UCSRA & _BV(UDRE)));
	UDR = c;
}

void hexStr(char * buf, uint16_t val) {
	sprintf(buf, "%04X", val); 
}
