#include "lcd.h"
#define F_CPU 16000000UL
#include <util/delay.h>

uint8_t switchedTo4BitMode = 0;
uint8_t checkBusyFlag = 0;

void lcdWriteCommand(uint8_t command) {
	if (switchedTo4BitMode){
  		checkBusyFlag = 1;
		lcdWrite(command, 0);
		checkBusyFlag = 0;
		lcdWrite(command << 4, 0);
	}else {
		lcdWrite(command, 0);
	}
}

void lcdWriteData(uint8_t data) {
	if (switchedTo4BitMode){
		checkBusyFlag = 1;
		lcdWrite(data, 1);
		checkBusyFlag = 0;
		lcdWrite(data << 4, 1);
	}else {
		lcdWrite(data, 1);
	}
}

uint8_t lcdReadAddress() {
	if (switchedTo4BitMode){
		return ((lcdRead(0) & 0xf0) | (lcdRead(0) >> 4));
	}else{
		return lcdRead(0);
	}
}

uint8_t lcdReadData(){
	if (switchedTo4BitMode){
		return ((lcdRead(1) & 0xf0) | (lcdRead(1) >> 4));
	}else{
		return lcdRead(1);
	}
}

void lcdWrite(uint8_t data, uint8_t rs) {
	if (checkBusyFlag) while (lcdReadAddress() & 0x80) _delay_us(10);
	
	// write mode, R/W low
	LCD_CONTROL_PORT &= ~_BV(LCD_RW);
#ifdef BITS4
	LCD_DATA_DDR |= 0xf0; //four upper pins as outputs
#else
	LCD_DATA_DDR = 0xff; //all data pins as outputs
#endif	
	//R/S
	if (rs) LCD_CONTROL_PORT |= _BV(LCD_RS); else PORTB &= ~_BV(LCD_RS);
	//command
#ifdef BITS4
	LCD_DATA_PORT &= data | 0x0f; //reset bits
	LCD_DATA_PORT |= data & 0xf0; //set bits
#else
	LCD_DATA_PORT = data;
#endif
	//EN high
	LCD_CONTROL_PORT |= _BV(LCD_EN);
	//wait 1 us
	_delay_us(1);
	//EN low
	LCD_CONTROL_PORT &= ~_BV(LCD_EN);
	//wait 1 us
	_delay_us(1);
}

uint8_t lcdRead(uint8_t rs) {
#ifdef BITS4
	LCD_DATA_DDR &= 0x0f; //four upper pins as inputs
#else
	LCD_DATA_DDR = 0x00; //all data pins as inputs
#endif
	//R/S
	if (rs) LCD_CONTROL_PORT |= _BV(LCD_RS); else LCD_CONTROL_PORT &= ~_BV(LCD_RS);
	//R/W high
	LCD_CONTROL_PORT |= _BV(LCD_RW);
	//EN high
	LCD_CONTROL_PORT |= _BV(LCD_EN);
	//wait 1 us
	_delay_us(1);
	//read
	uint8_t data = LCD_DATA_INPUT;
	//EN low
	LCD_CONTROL_PORT &= ~_BV(LCD_EN);
	//wait 1 us
	_delay_us(1);
	return data;
}

void lcdInit() {
	LCD_CONTROL_DDR |= _BV(LCD_EN)|_BV(LCD_RW)|_BV(LCD_RS);
	_delay_ms(15);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS); // set interface - 8-bit
	_delay_ms(4);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS); // set interface - 8-bit
	_delay_ms(1);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS); // set interface - 8-bit
#ifdef BITS4
	_delay_ms(1);
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET); // set interface - 4-bit
	_delay_ms(1);
	switchedTo4BitMode = 1;
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_2LINES); // set interface - 4-bit, 2 lines, 5x8 font
#else
	checkBusyFlag = 1;
	lcdWriteCommand(LCD_COMMAND_FUNCTION_SET | LCD_OPTION_8BITS | LCD_OPTION_2LINES); // set interface - 8-bit, 2 lines, 5x8 font
#endif
	lcdWriteCommand(LCD_COMMAND_ON); // off
	lcdWriteCommand(LCD_COMMAND_ON | LCD_OPTION_DISPLAY); // on
	lcdWriteCommand(LCD_COMMAND_ENTRY_MODE | LCD_OPTION_INCREMENT); // scroll
	lcdWriteCommand(LCD_COMMAND_CLEAR); // clear
}

void lcdString(char * buf) {
	while (*buf != 0) {
		lcdWriteData(*buf);
		buf++;
	}
}
