#include "lcd.h"
#include <avr/interrupt.h>
#include <avr/io.h>

char buf[80];
volatile uint8_t cursorState = 0x04;
volatile uint8_t i = 0;

void sendString();

void init_UART(void)
{
	UBRRL = 103; //9600 bps @ 16 MHz crystal
	UCSRB = _BV(RXEN)|_BV(RXCIE); //EX enable, RX int enable
	UCSRC = _BV(URSEL)|_BV(UCSZ1)|_BV(UCSZ0); //n,8,1
}

ISR(USART_RXC_vect) {
	uint8_t data = UDR;
	switch (data) {
		case 0x01: lcdWriteCommand(LCD_COMMAND_CLEAR);
			break;
		case 0x02:
			cursorState |= 0x02;
			lcdWriteCommand(LCD_COMMAND_ON | cursorState);
			break;
		case 0x03:
			cursorState &= ~0x02;
			lcdWriteCommand(LCD_COMMAND_ON | cursorState);
			break;
		case 0x04:
			cursorState |= 0x01;
			lcdWriteCommand(LCD_COMMAND_ON | cursorState);
			break;
		case 0x05:
			cursorState &= ~0x01;
			lcdWriteCommand(LCD_COMMAND_ON | cursorState);
			break;
		case 0x06:
			lcdWriteCommand(LCD_COMMAND_MOVE);
			break;
		case 0x07:
			lcdWriteCommand(LCD_COMMAND_MOVE | 0x04);
			break;
		case 0x08:
			lcdWriteCommand(LCD_COMMAND_MOVE | 0x08);
			break;
		case 0x09:
			lcdWriteCommand(LCD_COMMAND_MOVE | 0x08 | 0x04);
			break;
		case 0xfe:
			lcdString(buf);
			i = 0;
			break;
		default: buf[i++] = data;
	}
	if (data==0xff) {
		i = 0;
		sendString();
	}
}

int main (void) { 
	init_UART();
	sei();
	lcdInit();
	while(1);
	return 0;
}

void sendString() {
	lcdWriteCommand(0x01);
	uint8_t i = 0;
	while (buf[i]!=0) {
		lcdWriteData(buf[i]);
		i++;
	}
	lcdWriteCommand(0xC0);
	i++;
	while (buf[i]!=0xff) {
		lcdWriteData(buf[i]);
		i++;
	}
}
